@@ -8,7 +8,6 @@ use std::sync::Arc;
 
 use fs_err as fs;
 use rustc_hash::FxHashSet;
-use tempfile::{tempdir, TempDir};
 use tracing::debug;
 
 pub use archive::ArchiveId;
@@ -121,25 +120,25 @@ pub struct Cache {
     ///
     /// Included to ensure that the temporary directory exists for the length of the operation, but
     /// is dropped at the end as appropriate.
-    _temp_dir_drop: Option<Arc<TempDir>>,
+    _temp_dir_drop: Option<Arc<tempfile::TempDir>>,
 }
 
 impl Cache {
     /// A persistent cache directory at `root`.
-    pub fn from_path(root: impl Into<PathBuf>) -> Result<Self, io::Error> {
-        Ok(Self {
+    pub fn from_path(root: impl Into<PathBuf>) -> Self {
+        Self {
             root: root.into(),
-            refresh: Refresh::None,
+            refresh: Refresh::None(Timestamp::now()),
             _temp_dir_drop: None,
-        })
+        }
     }
 
     /// Create a temporary cache directory.
     pub fn temp() -> Result<Self, io::Error> {
-        let temp_dir = tempdir()?;
+        let temp_dir = tempfile::tempdir()?;
         Ok(Self {
             root: temp_dir.path().to_path_buf(),
-            refresh: Refresh::None,
+            refresh: Refresh::None(Timestamp::now()),
             _temp_dir_drop: Some(Arc::new(temp_dir)),
         })
     }
@@ -180,16 +179,25 @@ impl Cache {
         self.bucket(CacheBucket::Archive).join(id)
     }
 
+    /// Create an ephemeral Python environment in the cache.
+    pub fn environment(&self) -> io::Result<tempfile::TempDir> {
+        fs::create_dir_all(self.bucket(CacheBucket::Builds))?;
+        tempfile::tempdir_in(self.bucket(CacheBucket::Builds))
+    }
+
     /// Returns `true` if a cache entry must be revalidated given the [`Refresh`] policy.
     pub fn must_revalidate(&self, package: &PackageName) -> bool {
         match &self.refresh {
-            Refresh::None => false,
+            Refresh::None(_) => false,
             Refresh::All(_) => true,
             Refresh::Packages(packages, _) => packages.contains(package),
         }
     }
 
-    /// Returns `true` if a cache entry is up-to-date given the [`Refresh`] policy.
+    /// Returns the [`Freshness`] for a cache entry, validating it against the [`Refresh`] policy.
+    ///
+    /// A cache entry is considered fresh if it was created after the cache itself was
+    /// initialized, or if the [`Refresh`] policy does not require revalidation.
     pub fn freshness(
         &self,
         entry: &CacheEntry,
@@ -197,7 +205,7 @@ impl Cache {
     ) -> io::Result<Freshness> {
         // Grab the cutoff timestamp, if it's relevant.
         let timestamp = match &self.refresh {
-            Refresh::None => return Ok(Freshness::Fresh),
+            Refresh::None(_) => return Ok(Freshness::Fresh),
             Refresh::All(timestamp) => timestamp,
             Refresh::Packages(packages, timestamp) => {
                 if package.map_or(true, |package| packages.contains(package)) {
@@ -221,6 +229,26 @@ impl Cache {
         }
     }
 
+    /// Returns `true` if a cache entry is up-to-date. Unlike [`Cache::freshness`], this method does
+    /// not take the [`Refresh`] policy into account.
+    ///
+    /// A cache entry is considered up-to-date if it was created after the [`Cache`] instance itself
+    /// was initialized.
+    pub fn is_fresh(&self, entry: &CacheEntry) -> io::Result<bool> {
+        // Grab the cutoff timestamp.
+        let timestamp = match &self.refresh {
+            Refresh::None(timestamp) => timestamp,
+            Refresh::All(timestamp) => timestamp,
+            Refresh::Packages(_packages, timestamp) => timestamp,
+        };
+
+        match fs::metadata(entry.path()) {
+            Ok(metadata) => Ok(Timestamp::from_metadata(&metadata) >= *timestamp),
+            Err(err) if err.kind() == io::ErrorKind::NotFound => Ok(false),
+            Err(err) => Err(err),
+        }
+    }
+
     /// Persist a temporary directory to the artifact store, returning its unique ID.
     pub async fn persist(
         &self,
@@ -267,9 +295,9 @@ impl Cache {
         // Add an empty .gitignore to the build bucket, to ensure that the cache's own .gitignore
         // doesn't interfere with source distribution builds. Build backends (like hatchling) will
         // traverse upwards to look for .gitignore files.
-        fs::create_dir_all(root.join(CacheBucket::BuiltWheels.to_str()))?;
+        fs::create_dir_all(root.join(CacheBucket::SourceDistributions.to_str()))?;
         match fs::OpenOptions::new().write(true).create_new(true).open(
-            root.join(CacheBucket::BuiltWheels.to_str())
+            root.join(CacheBucket::SourceDistributions.to_str())
                 .join(".gitignore"),
         ) {
             Ok(_) => {}
@@ -284,10 +312,10 @@ impl Cache {
         // We have to put this below the gitignore. Otherwise, if the build backend uses the rust
         // ignore crate it will walk up to the top level .gitignore and ignore its python source
         // files.
-        fs::OpenOptions::new()
-            .create(true)
-            .write(true)
-            .open(root.join(CacheBucket::BuiltWheels.to_str()).join(".git"))?;
+        fs::OpenOptions::new().create(true).write(true).open(
+            root.join(CacheBucket::SourceDistributions.to_str())
+                .join(".git"),
+        )?;
 
         Ok(Self {
             root: fs::canonicalize(root)?,
@@ -343,7 +371,22 @@ impl Cache {
             }
         }
 
-        // Second, remove any unused archives (by searching for archives that are not symlinked).
+        // Second, remove any cached environments. These are never referenced by symlinks, so we can
+        // remove them directly.
+        match fs::read_dir(self.bucket(CacheBucket::Environments)) {
+            Ok(entries) => {
+                for entry in entries {
+                    let entry = entry?;
+                    let path = fs_err::canonicalize(entry.path())?;
+                    debug!("Removing dangling cache entry: {}", path.display());
+                    summary += rm_rf(path)?;
+                }
+            }
+            Err(err) if err.kind() == io::ErrorKind::NotFound => (),
+            Err(err) => return Err(err),
+        }
+
+        // Third, remove any unused archives (by searching for archives that are not symlinked).
         // TODO(charlie): Remove any unused source distributions. This requires introspecting the
         // cache contents, e.g., reading and deserializing the manifests.
         let mut references = FxHashSet::default();
@@ -354,19 +397,27 @@ impl Cache {
                 for entry in walkdir::WalkDir::new(bucket) {
                     let entry = entry?;
                     if entry.file_type().is_symlink() {
-                        references.insert(entry.path().canonicalize()?);
+                        if let Ok(target) = fs_err::canonicalize(entry.path()) {
+                            references.insert(target);
+                        }
                     }
                 }
             }
         }
 
-        for entry in fs::read_dir(self.bucket(CacheBucket::Archive))? {
-            let entry = entry?;
-            let path = entry.path().canonicalize()?;
-            if !references.contains(&path) {
-                debug!("Removing dangling cache entry: {}", path.display());
-                summary += rm_rf(path)?;
+        match fs::read_dir(self.bucket(CacheBucket::Archive)) {
+            Ok(entries) => {
+                for entry in entries {
+                    let entry = entry?;
+                    let path = fs_err::canonicalize(entry.path())?;
+                    if !references.contains(&path) {
+                        debug!("Removing dangling cache entry: {}", path.display());
+                        summary += rm_rf(path)?;
+                    }
+                }
             }
+            Err(err) if err.kind() == io::ErrorKind::NotFound => (),
+            Err(err) => return Err(err),
         }
 
         Ok(summary)
@@ -476,8 +527,8 @@ pub enum CacheBucket {
     ///                 └── flask-3.0.0.dist-info
     ///                     └── ...
     Wheels,
-    /// Wheels built from source distributions, their extracted metadata and the cache policy of
-    /// the source distribution.
+    /// Source distributions, wheels built from source distributions, their extracted metadata, and the
+    /// cache policy of the source distribution.
     ///
     /// The structure is similar of that of the `Wheel` bucket, except we have an additional layer
     /// for the source distribution filename and the metadata is at the source distribution-level,
@@ -486,6 +537,9 @@ pub enum CacheBucket {
     /// TODO(konstin): The cache policy should be on the source distribution level, the metadata we
     /// can put next to the wheels as in the `Wheels` bucket.
     ///
+    /// The unzipped source distribution is stored in a directory matching the source distribution
+    /// acrhive name.
+    ///
     /// Source distributions are built into zipped wheel files (as PEP 517 specifies) and unzipped
     /// lazily before installing. So when resolving, we only build the wheel and store the archive
     /// file in the cache, when installing, we unpack it under the same name (exclusive of the
@@ -517,32 +571,35 @@ pub enum CacheBucket {
     ///
     /// ...may be cached as:
     /// ```text
-    /// built-wheels-v0/
+    /// built-wheels-v3/
     /// ├── git
-    /// │   └── a67db8ed076e3814
-    /// │       └── 843b753e9e8cb74e83cac55598719b39a4d5ef1f
-    /// │           ├── manifest.msgpack
-    /// │           ├── metadata.msgpack
-    /// │           └── pydantic_extra_types-2.1.0-py3-none-any.whl
+    /// │   └── 2122faf3e081fb7a
+    /// │       └── 7a2d650a4a7b4d04
+    /// │           ├── metadata.msgpack
+    /// │           └── pydantic_extra_types-2.9.0-py3-none-any.whl
     /// ├── pypi
-    /// │   └── django
-    /// │       └── django-allauth-0.51.0.tar.gz
-    /// │           ├── django_allauth-0.51.0-py3-none-any.whl
-    /// │           ├── manifest.msgpack
-    /// │           └── metadata.msgpack
+    /// │   └── django-allauth
+    /// │       └── 0.51.0
+    /// │           ├── 0gH-_fwv8tdJ7JwwjJsUc
+    /// │           │   ├── django-allauth-0.51.0.tar.gz
+    /// │           │   │   └── [UNZIPPED CONTENTS]
+    /// │           │   ├── django_allauth-0.51.0-py3-none-any.whl
+    /// │           │   └── metadata.msgpack
+    /// │           └── revision.http
     /// └── url
     ///     └── 6781bd6440ae72c2
-    ///         └── werkzeug
-    ///             └── werkzeug-3.0.1.tar.gz
-    ///                 ├── manifest.msgpack
-    ///                 ├── metadata.msgpack
-    ///                 └── werkzeug-3.0.1-py3-none-any.whl
+    ///         ├── APYY01rbIfpAo_ij9sCY6
+    ///         │   ├── metadata.msgpack
+    ///         │   ├── werkzeug-3.0.1-py3-none-any.whl
+    ///         │   └── werkzeug-3.0.1.tar.gz
+    ///         │       └── [UNZIPPED CONTENTS]
+    ///         └── revision.http
     /// ```
     ///
     /// Structurally, the `manifest.msgpack` is empty, and only contains the caching information
     /// needed to invalidate the cache. The `metadata.msgpack` contains the metadata of the source
     /// distribution.
-    BuiltWheels,
+    SourceDistributions,
     /// Flat index responses, a format very similar to the simple metadata API.
     ///
     /// Cache structure:
@@ -605,18 +662,25 @@ pub enum CacheBucket {
     /// that cache entries can be atomically replaced and removed, as storing directories in the
     /// other buckets directly would make atomic operations impossible.
     Archive,
+    /// Ephemeral virtual environments used to execute PEP 517 builds and other operations.
+    Builds,
+    /// Reusable virtual environments used to invoke Python tools.
+    Environments,
 }
 
 impl CacheBucket {
     fn to_str(self) -> &'static str {
         match self {
-            Self::BuiltWheels => "built-wheels-v3",
+            // Note, next time we change the version we should change the name of this bucket to `source-dists-v0`
+            Self::SourceDistributions => "built-wheels-v3",
             Self::FlatIndex => "flat-index-v0",
             Self::Git => "git-v0",
-            Self::Interpreter => "interpreter-v1",
-            Self::Simple => "simple-v8",
+            Self::Interpreter => "interpreter-v2",
+            Self::Simple => "simple-v10",
             Self::Wheels => "wheels-v1",
             Self::Archive => "archive-v0",
+            Self::Builds => "builds-v0",
+            Self::Environments => "environments-v1",
         }
     }
 
@@ -642,8 +706,8 @@ impl CacheBucket {
                 let root = cache.bucket(self).join(WheelCacheKind::Pypi);
                 summary += rm_rf(root.join(name.to_string()))?;
 
-                // For alternate indices, we expect a directory for every index, followed by a
-                // directory per package (indexed by name).
+                // For alternate indices, we expect a directory for every index (under an `index`
+                // subdirectory), followed by a directory per package (indexed by name).
                 let root = cache.bucket(self).join(WheelCacheKind::Index);
                 for directory in directories(root) {
                     summary += rm_rf(directory.join(name.to_string()))?;
@@ -656,13 +720,13 @@ impl CacheBucket {
                     summary += rm_rf(directory.join(name.to_string()))?;
                 }
             }
-            Self::BuiltWheels => {
+            Self::SourceDistributions => {
                 // For `pypi` wheels, we expect a directory per package (indexed by name).
                 let root = cache.bucket(self).join(WheelCacheKind::Pypi);
                 summary += rm_rf(root.join(name.to_string()))?;
 
-                // For alternate indices, we expect a directory for every index, followed by a
-                // directory per package (indexed by name).
+                // For alternate indices, we expect a directory for every index (under an `index`
+                // subdirectory), followed by a directory per package (indexed by name).
                 let root = cache.bucket(self).join(WheelCacheKind::Index);
                 for directory in directories(root) {
                     summary += rm_rf(directory.join(name.to_string()))?;
@@ -705,9 +769,9 @@ impl CacheBucket {
                 let root = cache.bucket(self).join(WheelCacheKind::Pypi);
                 summary += rm_rf(root.join(format!("{name}.rkyv")))?;
 
-                // For alternate indices, we expect a directory for every index, followed by a
-                // MsgPack file per package, indexed by name.
-                let root = cache.bucket(self).join(WheelCacheKind::Url);
+                // For alternate indices, we expect a directory for every index (under an `index`
+                // subdirectory), followed by a directory per package (indexed by name).
+                let root = cache.bucket(self).join(WheelCacheKind::Index);
                 for directory in directories(root) {
                     summary += rm_rf(directory.join(format!("{name}.rkyv")))?;
                 }
@@ -727,20 +791,28 @@ impl CacheBucket {
             Self::Archive => {
                 // Nothing to do.
             }
+            Self::Builds => {
+                // Nothing to do.
+            }
+            Self::Environments => {
+                // Nothing to do.
+            }
         }
         Ok(summary)
     }
 
     /// Return an iterator over all cache buckets.
-    pub fn iter() -> impl Iterator<Item = CacheBucket> {
+    pub fn iter() -> impl Iterator<Item = Self> {
         [
-            CacheBucket::Wheels,
-            CacheBucket::BuiltWheels,
-            CacheBucket::FlatIndex,
-            CacheBucket::Git,
-            CacheBucket::Interpreter,
-            CacheBucket::Simple,
-            CacheBucket::Archive,
+            Self::Wheels,
+            Self::SourceDistributions,
+            Self::FlatIndex,
+            Self::Git,
+            Self::Interpreter,
+            Self::Simple,
+            Self::Archive,
+            Self::Builds,
+            Self::Environments,
         ]
         .iter()
         .copied()
@@ -897,10 +969,13 @@ impl Freshness {
 }
 
 /// A refresh policy for cache entries.
+///
+/// Each policy stores a timestamp, even if no entries are refreshed, to enable out-of-policy
+/// freshness checks via [`Cache::is_fresh`].
 #[derive(Debug, Clone)]
 pub enum Refresh {
     /// Don't refresh any entries.
-    None,
+    None(Timestamp),
     /// Refresh entries linked to the given packages, if created before the given timestamp.
     Packages(Vec<PackageName>, Timestamp),
     /// Refresh all entries created before the given timestamp.
@@ -910,14 +985,15 @@ pub enum Refresh {
 impl Refresh {
     /// Determine the refresh strategy to use based on the command-line arguments.
     pub fn from_args(refresh: Option<bool>, refresh_package: Vec<PackageName>) -> Self {
+        let timestamp = Timestamp::now();
         match refresh {
-            Some(true) => Self::All(Timestamp::now()),
-            Some(false) => Self::None,
+            Some(true) => Self::All(timestamp),
+            Some(false) => Self::None(timestamp),
             None => {
                 if refresh_package.is_empty() {
-                    Self::None
+                    Self::None(timestamp)
                 } else {
-                    Self::Packages(refresh_package, Timestamp::now())
+                    Self::Packages(refresh_package, timestamp)
                 }
             }
         }
@@ -925,6 +1001,6 @@ impl Refresh {
 
     /// Returns `true` if no packages should be reinstalled.
     pub fn is_none(&self) -> bool {
-        matches!(self, Self::None)
+        matches!(self, Self::None(_))
     }
 }
