@@ -1,4 +1,4 @@
-use std::path::PathBuf;
+use std::path::{Path, PathBuf};
 
 use futures::{FutureExt, StreamExt};
 use reqwest::Response;
@@ -7,7 +7,6 @@ use url::Url;
 
 use distribution_filename::DistFilename;
 use distribution_types::{File, FileLocation, FlatIndexLocation, IndexUrl};
-use pep508_rs::VerbatimUrl;
 use uv_cache::{Cache, CacheBucket};
 
 use crate::cached_client::{CacheControl, CachedClientError};
@@ -16,13 +15,24 @@ use crate::{Connectivity, Error, ErrorKind, RegistryClient};
 
 #[derive(Debug, thiserror::Error)]
 pub enum FlatIndexError {
+    #[error("Expected a file URL, but received: {0}")]
+    NonFileUrl(Url),
+
     #[error("Failed to read `--find-links` directory: {0}")]
-    FindLinksDirectory(PathBuf, #[source] std::io::Error),
+    FindLinksDirectory(PathBuf, #[source] FindLinksDirectoryError),
 
     #[error("Failed to read `--find-links` URL: {0}")]
     FindLinksUrl(Url, #[source] Error),
 }
 
+#[derive(Debug, thiserror::Error)]
+pub enum FindLinksDirectoryError {
+    #[error(transparent)]
+    Io(#[from] std::io::Error),
+    #[error(transparent)]
+    VerbatimUrl(#[from] pep508_rs::VerbatimUrlError),
+}
+
 #[derive(Debug, Default, Clone)]
 pub struct FlatIndexEntries {
     /// The list of `--find-links` entries.
@@ -89,12 +99,17 @@ impl<'a> FlatIndexClient<'a> {
         let mut fetches = futures::stream::iter(indexes)
             .map(|index| async move {
                 let entries = match index {
-                    FlatIndexLocation::Path(path) => Self::read_from_directory(path)
-                        .map_err(|err| FlatIndexError::FindLinksDirectory(path.clone(), err))?,
+                    FlatIndexLocation::Path(url) => {
+                        let path = url
+                            .to_file_path()
+                            .map_err(|()| FlatIndexError::NonFileUrl(url.to_url()))?;
+                        Self::read_from_directory(&path, index)
+                            .map_err(|err| FlatIndexError::FindLinksDirectory(path.clone(), err))?
+                    }
                     FlatIndexLocation::Url(url) => self
-                        .read_from_url(url)
+                        .read_from_url(url, index)
                         .await
-                        .map_err(|err| FlatIndexError::FindLinksUrl(url.clone(), err))?,
+                        .map_err(|err| FlatIndexError::FindLinksUrl(url.to_url(), err))?,
                 };
                 if entries.is_empty() {
                     warn!("No packages found in `--find-links` entry: {}", index);
@@ -118,7 +133,11 @@ impl<'a> FlatIndexClient<'a> {
     }
 
     /// Read a flat remote index from a `--find-links` URL.
-    async fn read_from_url(&self, url: &Url) -> Result<FlatIndexEntries, Error> {
+    async fn read_from_url(
+        &self,
+        url: &Url,
+        flat_index: &FlatIndexLocation,
+    ) -> Result<FlatIndexEntries, Error> {
         let cache_entry = self.cache.entry(
             CacheBucket::FlatIndex,
             "html",
@@ -181,14 +200,13 @@ impl<'a> FlatIndexClient<'a> {
             .await;
         match response {
             Ok(files) => {
-                let index_url = IndexUrl::Url(VerbatimUrl::from_url(url.clone()));
                 let files = files
                     .into_iter()
                     .filter_map(|file| {
                         Some((
                             DistFilename::try_from_normalized_filename(&file.filename)?,
                             file,
-                            index_url.clone(),
+                            IndexUrl::from(flat_index.clone()),
                         ))
                     })
                     .collect();
@@ -202,19 +220,32 @@ impl<'a> FlatIndexClient<'a> {
     }
 
     /// Read a flat remote index from a `--find-links` directory.
-    fn read_from_directory(path: &PathBuf) -> Result<FlatIndexEntries, std::io::Error> {
-        // Absolute paths are required for the URL conversion.
-        let path = fs_err::canonicalize(path)?;
-        let index_url = IndexUrl::Path(VerbatimUrl::from_path(&path));
-
+    fn read_from_directory(
+        path: &Path,
+        flat_index: &FlatIndexLocation,
+    ) -> Result<FlatIndexEntries, FindLinksDirectoryError> {
         let mut dists = Vec::new();
         for entry in fs_err::read_dir(path)? {
             let entry = entry?;
             let metadata = entry.metadata()?;
-            if !metadata.is_file() {
+
+            if metadata.is_dir() {
                 continue;
             }
 
+            if metadata.is_symlink() {
+                let Ok(target) = entry.path().read_link() else {
+                    warn!(
+                        "Skipping unreadable symlink in `--find-links` directory: {}",
+                        entry.path().display()
+                    );
+                    continue;
+                };
+                if target.is_dir() {
+                    continue;
+                }
+            }
+
             let Ok(filename) = entry.file_name().into_string() else {
                 warn!(
                     "Skipping non-UTF-8 filename in `--find-links` directory: {}",
@@ -241,7 +272,7 @@ impl<'a> FlatIndexClient<'a> {
                 );
                 continue;
             };
-            dists.push((filename, file, index_url.clone()));
+            dists.push((filename, file, IndexUrl::from(flat_index.clone())));
         }
         Ok(FlatIndexEntries::from_entries(dists))
     }
