@@ -1,9 +1,6 @@
 use std::fmt::Debug;
-use std::path::Path;
 
-use anyhow::Result;
 use same_file::is_same_file;
-use serde::Deserialize;
 use tracing::{debug, trace};
 use url::Url;
 
@@ -17,14 +14,16 @@ pub(crate) enum RequirementSatisfaction {
     Mismatch,
     Satisfied,
     OutOfDate,
-    Dynamic,
 }
 
 impl RequirementSatisfaction {
     /// Returns true if a requirement is satisfied by an installed distribution.
     ///
     /// Returns an error if IO fails during a freshness check for a local path.
-    pub(crate) fn check(distribution: &InstalledDist, source: &RequirementSource) -> Result<Self> {
+    pub(crate) fn check(
+        distribution: &InstalledDist,
+        source: &RequirementSource,
+    ) -> anyhow::Result<Self> {
         trace!(
             "Comparing installed with source: {:?} {:?}",
             distribution,
@@ -250,59 +249,8 @@ impl RequirementSatisfaction {
                     return Ok(Self::OutOfDate);
                 }
 
-                // Does the package have dynamic metadata?
-                if is_dynamic(requested_path) {
-                    trace!("Dependency is dynamic");
-                    return Ok(Self::Dynamic);
-                }
-
                 Ok(Self::Satisfied)
             }
         }
     }
 }
-
-/// Returns `true` if the source tree at the given path contains dynamic metadata.
-fn is_dynamic(path: &Path) -> bool {
-    // If there's no `pyproject.toml`, we assume it's dynamic.
-    let Ok(contents) = fs_err::read_to_string(path.join("pyproject.toml")) else {
-        return true;
-    };
-    let Ok(pyproject_toml) = toml::from_str::<PyProjectToml>(&contents) else {
-        return true;
-    };
-    // If `[project]` is not present, we assume it's dynamic.
-    let Some(project) = pyproject_toml.project else {
-        // ...unless it appears to be a Poetry project.
-        return pyproject_toml
-            .tool
-            .map_or(true, |tool| tool.poetry.is_none());
-    };
-    // `[project.dynamic]` must be present and non-empty.
-    project.dynamic.is_some_and(|dynamic| !dynamic.is_empty())
-}
-
-/// A pyproject.toml as specified in PEP 517.
-#[derive(Deserialize, Debug)]
-#[serde(rename_all = "kebab-case")]
-struct PyProjectToml {
-    project: Option<Project>,
-    tool: Option<Tool>,
-}
-
-#[derive(Deserialize, Debug)]
-#[serde(rename_all = "kebab-case")]
-struct Project {
-    dynamic: Option<Vec<String>>,
-}
-
-#[derive(Deserialize, Debug)]
-struct Tool {
-    poetry: Option<ToolPoetry>,
-}
-
-#[derive(Deserialize, Debug)]
-struct ToolPoetry {
-    #[allow(dead_code)]
-    name: Option<String>,
-}
