@@ -48,6 +48,7 @@ pub(crate) async fn pip_compile(
     requirements: &[RequirementsSource],
     constraints: &[RequirementsSource],
     overrides: &[RequirementsSource],
+    constraints_from_workspace: Vec<Requirement>,
     overrides_from_workspace: Vec<Requirement>,
     extras: ExtrasSpecification,
     output_file: Option<&Path>,
@@ -126,6 +127,12 @@ pub(crate) async fn pip_compile(
     )
     .await?;
 
+    let constraints = constraints
+        .iter()
+        .cloned()
+        .chain(constraints_from_workspace.into_iter())
+        .collect();
+
     let overrides: Vec<UnresolvedRequirementSpecification> = overrides
         .iter()
         .cloned()
