@@ -3,7 +3,7 @@ use std::borrow::Cow;
 use pep508_rs::MarkerEnvironment;
 use platform_tags::{Tags, TagsError};
 use uv_configuration::TargetTriple;
-use uv_toolchain::{Interpreter, PythonVersion};
+use uv_python::{Interpreter, PythonVersion};
 
 pub(crate) mod check;
 pub(crate) mod compile;
@@ -13,6 +13,7 @@ pub(crate) mod list;
 pub(crate) mod operations;
 pub(crate) mod show;
 pub(crate) mod sync;
+pub(crate) mod tree;
 pub(crate) mod uninstall;
 
 // Determine the tags, markers, and interpreter to use for resolution.
