@@ -5,16 +5,16 @@ use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, Connectivity, FlatIndexClient, RegistryClientBuilder};
 use uv_configuration::{Concurrency, ExtrasSpecification, PreviewMode, SetupPyStrategy};
 use uv_dispatch::BuildDispatch;
-use uv_distribution::pyproject::{DependencyType, Source, SourceError};
-use uv_distribution::pyproject_mut::PyProjectTomlMut;
-use uv_distribution::{DistributionDatabase, ProjectWorkspace, VirtualProject, Workspace};
+use uv_distribution::DistributionDatabase;
 use uv_normalize::PackageName;
 use uv_python::{PythonFetch, PythonPreference, PythonRequest};
-
 use uv_requirements::{NamedRequirementsResolver, RequirementsSource, RequirementsSpecification};
 use uv_resolver::FlatIndex;
 use uv_types::{BuildIsolation, HashStrategy};
 use uv_warnings::warn_user_once;
+use uv_workspace::pyproject::{DependencyType, Source, SourceError};
+use uv_workspace::pyproject_mut::PyProjectTomlMut;
+use uv_workspace::{DiscoveryOptions, ProjectWorkspace, VirtualProject, Workspace};
 
 use crate::commands::pip::operations::Modifications;
 use crate::commands::pip::resolution_environment;
@@ -54,12 +54,12 @@ pub(crate) async fn add(
 
     // Find the project in the workspace.
     let project = if let Some(package) = package {
-        Workspace::discover(&std::env::current_dir()?, None)
+        Workspace::discover(&std::env::current_dir()?, &DiscoveryOptions::default())
             .await?
             .with_current_project(package.clone())
             .with_context(|| format!("Package `{package}` not found in workspace"))?
     } else {
-        ProjectWorkspace::discover(&std::env::current_dir()?, None).await?
+        ProjectWorkspace::discover(&std::env::current_dir()?, &DiscoveryOptions::default()).await?
     };
 
     // Discover or create the virtual environment.
@@ -239,7 +239,7 @@ pub(crate) async fn add(
         &VirtualProject::Project(project),
         &venv,
         &lock,
-        extras,
+        &extras,
         dev,
         Modifications::Sufficient,
         settings.as_ref().into(),
