@@ -1,18 +1,20 @@
 #![allow(clippy::single_match_else)]
 
-use anstream::eprint;
-use distribution_types::{Diagnostic, UnresolvedRequirementSpecification, VersionId};
-use owo_colors::OwoColorize;
-use pep440_rs::Version;
-use rustc_hash::{FxBuildHasher, FxHashMap};
 use std::collections::BTreeSet;
 use std::{fmt::Write, path::Path};
+
+use anstream::eprint;
+use owo_colors::OwoColorize;
+use rustc_hash::{FxBuildHasher, FxHashMap};
 use tracing::debug;
+
+use distribution_types::{Diagnostic, UnresolvedRequirementSpecification, VersionId};
+use pep440_rs::Version;
 use uv_cache::Cache;
 use uv_client::{Connectivity, FlatIndexClient, RegistryClientBuilder};
 use uv_configuration::{Concurrency, ExtrasSpecification, PreviewMode, Reinstall, SetupPyStrategy};
 use uv_dispatch::BuildDispatch;
-use uv_distribution::{Workspace, DEV_DEPENDENCIES};
+use uv_distribution::DEV_DEPENDENCIES;
 use uv_git::ResolvedRepositoryReference;
 use uv_normalize::PackageName;
 use uv_python::{Interpreter, PythonFetch, PythonPreference, PythonRequest};
@@ -22,6 +24,7 @@ use uv_resolver::{
 };
 use uv_types::{BuildIsolation, EmptyInstalledPackages, HashStrategy};
 use uv_warnings::{warn_user, warn_user_once};
+use uv_workspace::{DiscoveryOptions, Workspace};
 
 use crate::commands::project::{find_requires_python, FoundInterpreter, ProjectError, SharedState};
 use crate::commands::{pip, ExitStatus};
@@ -48,7 +51,8 @@ pub(crate) async fn lock(
     }
 
     // Find the project requirements.
-    let workspace = Workspace::discover(&std::env::current_dir()?, None).await?;
+    let workspace =
+        Workspace::discover(&std::env::current_dir()?, &DiscoveryOptions::default()).await?;
 
     // Find an interpreter for the project
     let interpreter = FoundInterpreter::discover(
@@ -208,7 +212,7 @@ pub(super) async fn do_lock(
         .into_iter()
         .map(UnresolvedRequirementSpecification::from)
         .collect::<Vec<_>>();
-    let constraints = vec![];
+    let constraints = workspace.constraints();
     let dev = vec![DEV_DEPENDENCIES.clone()];
     let source_trees = vec![];
 
@@ -220,13 +224,13 @@ pub(super) async fn do_lock(
         if requires_python.is_unbounded() {
             let default =
                 RequiresPython::greater_than_equal_version(&interpreter.python_minor_version());
-            warn_user!("The workspace `requires-python` field does not contain a lower bound: `{requires_python}`. Set a lower bound to indicate the minimum compatible Python version (e.g., `{default}`).");
+            warn_user!("The workspace `requires-python` value does not contain a lower bound: `{requires_python}`. Set a lower bound to indicate the minimum compatible Python version (e.g., `{default}`).");
         }
         requires_python
     } else {
         let default =
             RequiresPython::greater_than_equal_version(&interpreter.python_minor_version());
-        warn_user!("No `requires-python` field found in the workspace. Defaulting to `{default}`.");
+        warn_user!("No `requires-python` value found in the workspace. Defaulting to `{default}`.");
         default
     };
 
@@ -264,6 +268,58 @@ pub(super) async fn do_lock(
         FlatIndex::from_entries(entries, None, &hasher, build_options)
     };
 
+    // If any of the resolution-determining settings changed, invalidate the lock.
+    let existing_lock = existing_lock.filter(|lock| {
+        if lock.resolution_mode() != options.resolution_mode {
+            let _ = writeln!(
+                printer.stderr(),
+                "Ignoring existing lockfile due to change in resolution mode: `{}` vs. `{}`",
+                lock.resolution_mode().cyan(),
+                options.resolution_mode.cyan()
+            );
+            return false;
+        }
+        if lock.prerelease_mode() != options.prerelease_mode {
+            let _ = writeln!(
+                printer.stderr(),
+                "Ignoring existing lockfile due to change in prerelease mode: `{}` vs. `{}`",
+                lock.prerelease_mode().cyan(),
+                options.prerelease_mode.cyan()
+            );
+            return false;
+        }
+        match (lock.exclude_newer(), options.exclude_newer) {
+            (None, None) => (),
+            (Some(existing), Some(provided)) if existing == provided => (),
+            (Some(existing), Some(provided)) => {
+                let _ = writeln!(
+                    printer.stderr(),
+                    "Ignoring existing lockfile due to change in timestamp cutoff: `{}` vs. `{}`",
+                    existing.cyan(),
+                    provided.cyan()
+                );
+                return false;
+            }
+            (Some(existing), None) => {
+                let _ = writeln!(
+                    printer.stderr(),
+                    "Ignoring existing lockfile due to removal of timestamp cutoff: `{}`",
+                    existing.cyan(),
+                );
+                return false;
+            }
+            (None, Some(provided)) => {
+                let _ = writeln!(
+                    printer.stderr(),
+                    "Ignoring existing lockfile due to addition of timestamp cutoff: `{}`",
+                    provided.cyan()
+                );
+                return false;
+            }
+        }
+        true
+    });
+
     // If an existing lockfile exists, build up a set of preferences.
     let LockedRequirements { preferences, git } = existing_lock
         .as_ref()
