@@ -1,9 +1,9 @@
 use std::borrow::Cow;
 use std::ffi::OsString;
 use std::fmt::Write;
-use std::path::PathBuf;
+use std::path::{Path, PathBuf};
 
-use anyhow::{Context, Result};
+use anyhow::{bail, Context, Result};
 use itertools::Itertools;
 use owo_colors::OwoColorize;
 use tokio::process::Command;
@@ -14,7 +14,6 @@ use uv_cache::Cache;
 use uv_cli::ExternalCommand;
 use uv_client::{BaseClientBuilder, Connectivity};
 use uv_configuration::{Concurrency, ExtrasSpecification, PreviewMode};
-use uv_distribution::{VirtualProject, Workspace, WorkspaceError};
 use uv_fs::Simplified;
 use uv_installer::{SatisfiesResult, SitePackages};
 use uv_normalize::PackageName;
@@ -22,9 +21,9 @@ use uv_python::{
     request_from_version_file, EnvironmentPreference, Interpreter, PythonEnvironment, PythonFetch,
     PythonInstallation, PythonPreference, PythonRequest, VersionRequest,
 };
-
 use uv_requirements::{RequirementsSource, RequirementsSpecification};
 use uv_warnings::warn_user_once;
+use uv_workspace::{DiscoveryOptions, VirtualProject, Workspace, WorkspaceError};
 
 use crate::commands::pip::operations::Modifications;
 use crate::commands::project::environment::CachedEnvironment;
@@ -60,6 +59,28 @@ pub(crate) async fn run(
         warn_user_once!("`uv run` is experimental and may change without warning");
     }
 
+    // These cases seem quite complex because (in theory) they should change the "current package".
+    // Let's ban them entirely for now.
+    for source in &requirements {
+        match source {
+            RequirementsSource::PyprojectToml(_) => {
+                bail!("Adding requirements from a `pyproject.toml` is not supported in `uv run`");
+            }
+            RequirementsSource::SetupPy(_) => {
+                bail!("Adding requirements from a `setup.py` is not supported in `uv run`");
+            }
+            RequirementsSource::SetupCfg(_) => {
+                bail!("Adding requirements from a `setup.cfg` is not supported in `uv run`");
+            }
+            RequirementsSource::RequirementsTxt(path) => {
+                if path == Path::new("-") {
+                    bail!("Reading requirements from stdin is not supported in `uv run`");
+                }
+            }
+            _ => {}
+        }
+    }
+
     // Parse the input command.
     let command = RunCommand::from(command);
 
@@ -145,13 +166,15 @@ pub(crate) async fn run(
             // We need a workspace, but we don't need to have a current package, we can be e.g. in
             // the root of a virtual workspace and then switch into the selected package.
             Some(VirtualProject::Project(
-                Workspace::discover(&std::env::current_dir()?, None)
+                Workspace::discover(&std::env::current_dir()?, &DiscoveryOptions::default())
                     .await?
                     .with_current_project(package.clone())
                     .with_context(|| format!("Package `{package}` not found in workspace"))?,
             ))
         } else {
-            match VirtualProject::discover(&std::env::current_dir()?, None).await {
+            match VirtualProject::discover(&std::env::current_dir()?, &DiscoveryOptions::default())
+                .await
+            {
                 Ok(project) => Some(project),
                 Err(WorkspaceError::MissingPyprojectToml) => None,
                 Err(WorkspaceError::NonWorkspace(_)) => None,
@@ -215,7 +238,7 @@ pub(crate) async fn run(
                 &project,
                 &venv,
                 &lock,
-                extras,
+                &extras,
                 dev,
                 Modifications::Sufficient,
                 settings.as_ref().into(),
@@ -263,7 +286,7 @@ pub(crate) async fn run(
         );
     }
 
-    // Read the `--with` requirements.
+    // Read the requirements.
     let spec = if requirements.is_empty() {
         None
     } else {
@@ -281,6 +304,7 @@ pub(crate) async fn run(
     // any `--with` requirements, and we already have a base environment, then there's no need to
     // create an additional environment.
     let skip_ephemeral = base_interpreter.as_ref().is_some_and(|base_interpreter| {
+        // No additional requirements.
         let Some(spec) = spec.as_ref() else {
             return true;
         };
@@ -363,35 +387,28 @@ pub(crate) async fn run(
             false,
         )?;
 
-        if requirements.is_empty() {
-            Some(venv)
-        } else {
-            debug!("Syncing ephemeral requirements");
-
-            let client_builder = BaseClientBuilder::new()
-                .connectivity(connectivity)
-                .native_tls(native_tls);
-
-            let spec =
-                RequirementsSpecification::from_simple_sources(&requirements, &client_builder)
-                    .await?;
-
-            // Install the ephemeral requirements.
-            Some(
-                project::update_environment(
-                    venv,
-                    spec,
-                    &settings,
-                    &state,
-                    preview,
-                    connectivity,
-                    concurrency,
-                    native_tls,
-                    cache,
-                    printer,
+        match spec {
+            None => Some(venv),
+            Some(spec) if spec.is_empty() => Some(venv),
+            Some(spec) => {
+                debug!("Syncing ephemeral requirements");
+                // Install the ephemeral requirements.
+                Some(
+                    project::update_environment(
+                        venv,
+                        spec,
+                        &settings,
+                        &state,
+                        preview,
+                        connectivity,
+                        concurrency,
+                        native_tls,
+                        cache,
+                        printer,
+                    )
+                    .await?,
                 )
-                .await?,
-            )
+            }
         }
     };
 
