@@ -6,13 +6,13 @@ use uv_configuration::{
     Concurrency, ExtrasSpecification, HashCheckingMode, PreviewMode, SetupPyStrategy,
 };
 use uv_dispatch::BuildDispatch;
-use uv_distribution::{VirtualProject, DEV_DEPENDENCIES};
+use uv_distribution::DEV_DEPENDENCIES;
 use uv_installer::SitePackages;
 use uv_python::{PythonEnvironment, PythonFetch, PythonPreference, PythonRequest};
-
 use uv_resolver::{FlatIndex, Lock};
 use uv_types::{BuildIsolation, HashStrategy};
 use uv_warnings::warn_user_once;
+use uv_workspace::{DiscoveryOptions, VirtualProject};
 
 use crate::commands::pip::operations::Modifications;
 use crate::commands::project::lock::do_safe_lock;
@@ -45,7 +45,8 @@ pub(crate) async fn sync(
     }
 
     // Identify the project
-    let project = VirtualProject::discover(&std::env::current_dir()?, None).await?;
+    let project =
+        VirtualProject::discover(&std::env::current_dir()?, &DiscoveryOptions::default()).await?;
 
     // Discover or create the virtual environment.
     let venv = project::get_or_init_environment(
@@ -95,7 +96,7 @@ pub(crate) async fn sync(
         &project,
         &venv,
         &lock,
-        extras,
+        &extras,
         dev,
         modifications,
         settings.as_ref().into(),
@@ -117,7 +118,7 @@ pub(super) async fn do_sync(
     project: &VirtualProject,
     venv: &PythonEnvironment,
     lock: &Lock,
-    extras: ExtrasSpecification,
+    extras: &ExtrasSpecification,
     dev: bool,
     modifications: Modifications,
     settings: InstallerSettingsRef<'_>,
@@ -163,7 +164,7 @@ pub(super) async fn do_sync(
     let tags = venv.interpreter().tags()?;
 
     // Read the lockfile.
-    let resolution = lock.to_resolution(project, markers, tags, &extras, &dev)?;
+    let resolution = lock.to_resolution(project, markers, tags, extras, &dev)?;
 
     // Initialize the registry client.
     let client = RegistryClientBuilder::new(cache.clone())
