@@ -1,17 +1,16 @@
 use std::fmt::Write;
 
 use anyhow::Result;
-
 use indexmap::IndexMap;
 use owo_colors::OwoColorize;
+
 use pep508_rs::PackageName;
 use uv_cache::Cache;
 use uv_client::Connectivity;
 use uv_configuration::{Concurrency, PreviewMode};
-use uv_distribution::Workspace;
 use uv_python::{PythonFetch, PythonPreference, PythonRequest};
-
 use uv_warnings::warn_user_once;
+use uv_workspace::{DiscoveryOptions, Workspace};
 
 use crate::commands::pip::tree::DisplayDependencyGraph;
 use crate::commands::project::FoundInterpreter;
@@ -31,6 +30,7 @@ pub(crate) async fn tree(
     package: Vec<PackageName>,
     no_dedupe: bool,
     invert: bool,
+    show_version_specifiers: bool,
     python: Option<String>,
     settings: ResolverSettings,
     python_preference: PythonPreference,
@@ -47,7 +47,8 @@ pub(crate) async fn tree(
     }
 
     // Find the project requirements.
-    let workspace = Workspace::discover(&std::env::current_dir()?, None).await?;
+    let workspace =
+        Workspace::discover(&std::env::current_dir()?, &DiscoveryOptions::default()).await?;
 
     // Find an interpreter for the project
     let interpreter = FoundInterpreter::discover(
@@ -95,6 +96,7 @@ pub(crate) async fn tree(
         package,
         no_dedupe,
         invert,
+        show_version_specifiers,
         interpreter.markers(),
         packages,
     )
