@@ -37,16 +37,20 @@ fn help() {
           --offline
               Disable network access, relying only on locally cached data and locally available files
           --python-preference <PYTHON_PREFERENCE>
-              Whether to prefer using Python from uv or on the system [possible values: only-managed,
-              installed, managed, system, only-system]
+              Whether to prefer using Python installations that are already present on the system, or
+              those that are downloaded and installed by uv [possible values: only-managed, installed,
+              managed, system, only-system]
           --python-fetch <PYTHON_FETCH>
               Whether to automatically download Python when required [possible values: automatic,
               manual]
           --isolated
               Avoid discovering a `pyproject.toml` or `uv.toml` file in the current directory or any
               parent directories
+          --no-progress
+              Hides all progress outputs when set
       -n, --no-cache
-              Avoid reading from or writing to the cache [env: UV_NO_CACHE=]
+              Avoid reading from or writing to the cache, instead using a temporary directory for the
+              duration of the operation [env: UV_NO_CACHE=]
           --cache-dir [CACHE_DIR]
               Path to the cache directory [env: UV_CACHE_DIR=]
           --config-file <CONFIG_FILE>
@@ -96,16 +100,20 @@ fn help_flag() {
           --offline
               Disable network access, relying only on locally cached data and locally available files
           --python-preference <PYTHON_PREFERENCE>
-              Whether to prefer using Python from uv or on the system [possible values: only-managed,
-              installed, managed, system, only-system]
+              Whether to prefer using Python installations that are already present on the system, or
+              those that are downloaded and installed by uv [possible values: only-managed, installed,
+              managed, system, only-system]
           --python-fetch <PYTHON_FETCH>
               Whether to automatically download Python when required [possible values: automatic,
               manual]
           --isolated
               Avoid discovering a `pyproject.toml` or `uv.toml` file in the current directory or any
               parent directories
+          --no-progress
+              Hides all progress outputs when set
       -n, --no-cache
-              Avoid reading from or writing to the cache [env: UV_NO_CACHE=]
+              Avoid reading from or writing to the cache, instead using a temporary directory for the
+              duration of the operation [env: UV_NO_CACHE=]
           --cache-dir [CACHE_DIR]
               Path to the cache directory [env: UV_CACHE_DIR=]
           --config-file <CONFIG_FILE>
@@ -154,16 +162,20 @@ fn help_short_flag() {
           --offline
               Disable network access, relying only on locally cached data and locally available files
           --python-preference <PYTHON_PREFERENCE>
-              Whether to prefer using Python from uv or on the system [possible values: only-managed,
-              installed, managed, system, only-system]
+              Whether to prefer using Python installations that are already present on the system, or
+              those that are downloaded and installed by uv [possible values: only-managed, installed,
+              managed, system, only-system]
           --python-fetch <PYTHON_FETCH>
               Whether to automatically download Python when required [possible values: automatic,
               manual]
           --isolated
               Avoid discovering a `pyproject.toml` or `uv.toml` file in the current directory or any
               parent directories
+          --no-progress
+              Hides all progress outputs when set
       -n, --no-cache
-              Avoid reading from or writing to the cache [env: UV_NO_CACHE=]
+              Avoid reading from or writing to the cache, instead using a temporary directory for the
+              duration of the operation [env: UV_NO_CACHE=]
           --cache-dir [CACHE_DIR]
               Path to the cache directory [env: UV_CACHE_DIR=]
           --config-file <CONFIG_FILE>
@@ -223,8 +235,8 @@ fn help_subcommand() {
           --native-tls
               Whether to load TLS certificates from the platform's native certificate store.
               
-              By default, `uv` loads certificates from the bundled `webpki-roots` crate. The
-              `webpki-roots` are a reliable set of trust roots from Mozilla, and including them in `uv`
+              By default, uv loads certificates from the bundled `webpki-roots` crate. The
+              `webpki-roots` are a reliable set of trust roots from Mozilla, and including them in uv
               improves portability and performance (especially on macOS).
               
               However, in some cases, you may want to use the platform's native certificate store,
@@ -237,7 +249,8 @@ fn help_subcommand() {
               Disable network access, relying only on locally cached data and locally available files
 
           --python-preference <PYTHON_PREFERENCE>
-              Whether to prefer using Python from uv or on the system
+              Whether to prefer using Python installations that are already present on the system, or
+              those that are downloaded and installed by uv
 
               Possible values:
               - only-managed: Only use managed Python installations; never use system Python
@@ -262,8 +275,12 @@ fn help_subcommand() {
               Avoid discovering a `pyproject.toml` or `uv.toml` file in the current directory or any
               parent directories
 
+          --no-progress
+              Hides all progress outputs when set
+
       -n, --no-cache
-              Avoid reading from or writing to the cache
+              Avoid reading from or writing to the cache, instead using a temporary directory for the
+              duration of the operation
               
               [env: UV_NO_CACHE=]
 
@@ -340,8 +357,8 @@ fn help_subsubcommand() {
           --native-tls
               Whether to load TLS certificates from the platform's native certificate store.
               
-              By default, `uv` loads certificates from the bundled `webpki-roots` crate. The
-              `webpki-roots` are a reliable set of trust roots from Mozilla, and including them in `uv`
+              By default, uv loads certificates from the bundled `webpki-roots` crate. The
+              `webpki-roots` are a reliable set of trust roots from Mozilla, and including them in uv
               improves portability and performance (especially on macOS).
               
               However, in some cases, you may want to use the platform's native certificate store,
@@ -354,7 +371,8 @@ fn help_subsubcommand() {
               Disable network access, relying only on locally cached data and locally available files
 
           --python-preference <PYTHON_PREFERENCE>
-              Whether to prefer using Python from uv or on the system
+              Whether to prefer using Python installations that are already present on the system, or
+              those that are downloaded and installed by uv
 
               Possible values:
               - only-managed: Only use managed Python installations; never use system Python
@@ -379,8 +397,12 @@ fn help_subsubcommand() {
               Avoid discovering a `pyproject.toml` or `uv.toml` file in the current directory or any
               parent directories
 
+          --no-progress
+              Hides all progress outputs when set
+
       -n, --no-cache
-              Avoid reading from or writing to the cache
+              Avoid reading from or writing to the cache, instead using a temporary directory for the
+              duration of the operation
               
               [env: UV_NO_CACHE=]
 
@@ -441,16 +463,20 @@ fn help_flag_subcommand() {
           --offline
               Disable network access, relying only on locally cached data and locally available files
           --python-preference <PYTHON_PREFERENCE>
-              Whether to prefer using Python from uv or on the system [possible values: only-managed,
-              installed, managed, system, only-system]
+              Whether to prefer using Python installations that are already present on the system, or
+              those that are downloaded and installed by uv [possible values: only-managed, installed,
+              managed, system, only-system]
           --python-fetch <PYTHON_FETCH>
               Whether to automatically download Python when required [possible values: automatic,
               manual]
           --isolated
               Avoid discovering a `pyproject.toml` or `uv.toml` file in the current directory or any
               parent directories
+          --no-progress
+              Hides all progress outputs when set
       -n, --no-cache
-              Avoid reading from or writing to the cache [env: UV_NO_CACHE=]
+              Avoid reading from or writing to the cache, instead using a temporary directory for the
+              duration of the operation [env: UV_NO_CACHE=]
           --cache-dir [CACHE_DIR]
               Path to the cache directory [env: UV_CACHE_DIR=]
           --config-file <CONFIG_FILE>
@@ -496,16 +522,20 @@ fn help_flag_subsubcommand() {
           --offline
               Disable network access, relying only on locally cached data and locally available files
           --python-preference <PYTHON_PREFERENCE>
-              Whether to prefer using Python from uv or on the system [possible values: only-managed,
-              installed, managed, system, only-system]
+              Whether to prefer using Python installations that are already present on the system, or
+              those that are downloaded and installed by uv [possible values: only-managed, installed,
+              managed, system, only-system]
           --python-fetch <PYTHON_FETCH>
               Whether to automatically download Python when required [possible values: automatic,
               manual]
           --isolated
               Avoid discovering a `pyproject.toml` or `uv.toml` file in the current directory or any
               parent directories
+          --no-progress
+              Hides all progress outputs when set
       -n, --no-cache
-              Avoid reading from or writing to the cache [env: UV_NO_CACHE=]
+              Avoid reading from or writing to the cache, instead using a temporary directory for the
+              duration of the operation [env: UV_NO_CACHE=]
           --cache-dir [CACHE_DIR]
               Path to the cache directory [env: UV_CACHE_DIR=]
           --config-file <CONFIG_FILE>
@@ -608,16 +638,20 @@ fn help_with_global_option() {
           --offline
               Disable network access, relying only on locally cached data and locally available files
           --python-preference <PYTHON_PREFERENCE>
-              Whether to prefer using Python from uv or on the system [possible values: only-managed,
-              installed, managed, system, only-system]
+              Whether to prefer using Python installations that are already present on the system, or
+              those that are downloaded and installed by uv [possible values: only-managed, installed,
+              managed, system, only-system]
           --python-fetch <PYTHON_FETCH>
               Whether to automatically download Python when required [possible values: automatic,
               manual]
           --isolated
               Avoid discovering a `pyproject.toml` or `uv.toml` file in the current directory or any
               parent directories
+          --no-progress
+              Hides all progress outputs when set
       -n, --no-cache
-              Avoid reading from or writing to the cache [env: UV_NO_CACHE=]
+              Avoid reading from or writing to the cache, instead using a temporary directory for the
+              duration of the operation [env: UV_NO_CACHE=]
           --cache-dir [CACHE_DIR]
               Path to the cache directory [env: UV_CACHE_DIR=]
           --config-file <CONFIG_FILE>
@@ -700,16 +734,20 @@ fn test_with_no_pager() {
           --offline
               Disable network access, relying only on locally cached data and locally available files
           --python-preference <PYTHON_PREFERENCE>
-              Whether to prefer using Python from uv or on the system [possible values: only-managed,
-              installed, managed, system, only-system]
+              Whether to prefer using Python installations that are already present on the system, or
+              those that are downloaded and installed by uv [possible values: only-managed, installed,
+              managed, system, only-system]
           --python-fetch <PYTHON_FETCH>
               Whether to automatically download Python when required [possible values: automatic,
               manual]
           --isolated
               Avoid discovering a `pyproject.toml` or `uv.toml` file in the current directory or any
               parent directories
+          --no-progress
+              Hides all progress outputs when set
       -n, --no-cache
-              Avoid reading from or writing to the cache [env: UV_NO_CACHE=]
+              Avoid reading from or writing to the cache, instead using a temporary directory for the
+              duration of the operation [env: UV_NO_CACHE=]
           --cache-dir [CACHE_DIR]
               Path to the cache directory [env: UV_CACHE_DIR=]
           --config-file <CONFIG_FILE>
